@@ -43,6 +43,9 @@
  * Another important feature of the filter model is also that the rows 
  * need not be in the same order as the original rows in the back end model.
  *
+ * There is a suite of filters shipped with Dee which you can browse in the
+ * <link linkend="dee-1.0-Filters.top_of_page">Filters section</link>.
+ *
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -52,7 +55,7 @@
 #include "dee-model.h"
 #include "dee-proxy-model.h"
 #include "dee-filter-model.h"
-#include "dee-versioned-model.h"
+#include "dee-serializable-model.h"
 #include "dee-sequence-model.h"
 #include "dee-marshal.h"
 #include "trace-log.h"
@@ -102,24 +105,21 @@ enum
 /*
  * DeeModel forward declarations
  */
-static void           dee_filter_model_set_n_columns  (DeeModel *self,
-                                                       guint     n_columns);
-
-static void           dee_filter_model_set_column_type(DeeModel *self,
-                                                       guint     column,
-                                                       GType     type);
+static void           dee_filter_model_set_schema_full  (DeeModel           *self,
+                                                         const gchar* const *schema,
+                                                         guint               n_columns);
 
 static guint          dee_filter_model_get_n_rows     (DeeModel *self);
 
-static DeeModelIter*  dee_filter_model_append_valist  (DeeModel *self,
-                                                       va_list    args);
+static DeeModelIter*  dee_filter_model_append_row  (DeeModel  *self,
+                                                    GVariant **row_members);
 
-static DeeModelIter*  dee_filter_model_prepend_valist (DeeModel *self,
-                                                       va_list    args);
+static DeeModelIter*  dee_filter_model_prepend_row (DeeModel *self,
+                                                    GVariant **row_members);
 
-static DeeModelIter*  dee_filter_model_insert_before_valist (DeeModel     *self,
-                                                             DeeModelIter *iter,
-                                                             va_list    args);
+static DeeModelIter*  dee_filter_model_insert_row_before (DeeModel     *self,
+                                                          DeeModelIter *iter,
+                                                          GVariant **row_members);
 
 static void           dee_filter_model_remove         (DeeModel     *self,
                                                        DeeModelIter *iter);
@@ -138,7 +138,7 @@ static DeeModelIter* dee_filter_model_prev            (DeeModel     *self,
 static gboolean      dee_filter_model_is_first        (DeeModel     *self,
                                                        DeeModelIter *iter);
 
-static gint           dee_filter_model_get_position   (DeeModel     *self,
+static guint         dee_filter_model_get_position    (DeeModel     *self,
                                                        DeeModelIter *iter);
 
 /* Private forward declarations */
@@ -315,13 +315,11 @@ static void
 dee_filter_model_model_iface_init (DeeModelIface *iface)
 {
   /* Override column spec setters as that would cause a mess... */
-  iface->set_column_type      = dee_filter_model_set_column_type;
-  iface->set_n_columns        = dee_filter_model_set_n_columns;
-  
+  iface->set_schema_full      = dee_filter_model_set_schema_full;
   iface->get_n_rows           = dee_filter_model_get_n_rows;
-  iface->prepend_valist       = dee_filter_model_prepend_valist;
-  iface->append_valist        = dee_filter_model_append_valist;
-  iface->insert_before_valist = dee_filter_model_insert_before_valist;
+  iface->prepend_row          = dee_filter_model_prepend_row;
+  iface->append_row           = dee_filter_model_append_row;
+  iface->insert_row_before    = dee_filter_model_insert_row_before;
   iface->remove               = dee_filter_model_remove;
   iface->get_first_iter       = dee_filter_model_get_first_iter;
   iface->get_iter_at_row      = dee_filter_model_get_iter_at_row;
@@ -329,10 +327,6 @@ dee_filter_model_model_iface_init (DeeModelIface *iface)
   iface->prev                 = dee_filter_model_prev;
   iface->is_first             = dee_filter_model_is_first;
   iface->get_position         = dee_filter_model_get_position;
-  
-  // FIXME: freeze/thaw of filter model?
-//  iface->freeze_signals       = dee_filter_model_freeze_signals;
-//  iface->thaw_signals         = dee_filter_model_thaw_signals;
 }
 
 /*
@@ -348,7 +342,7 @@ dee_filter_model_model_iface_init (DeeModelIface *iface)
  * @orig_model: The back end model. This will be set as the
  *              #DeeProxyModel:back-end property
  *
- * Returns: A newly allocated #DeeFilterModel. Free with g_object_unref().
+ * Returns: (transfer full): A newly allocated #DeeFilterModel. Free with g_object_unref().
  */
 DeeModel*
 dee_filter_model_new (const DeeFilter *filter,
@@ -374,7 +368,7 @@ dee_filter_model_new (const DeeFilter *filter,
 /**
  * dee_filter_model_contains:
  * @self: The #DeeFilterModel to check
- * @iter: The #DeeModelIter to check
+ * @iter: (transfer none):The #DeeModelIter to check
  *
  * Check if @iter from the back end model is mapped in @self.
  * 
@@ -399,6 +393,8 @@ dee_filter_model_contains (DeeFilterModel *self,
  *
  * This method is usually called when implementing #DeeModelMapFunc or
  * #DeeModelMapNotify methods.
+ *
+ * Return value: (transfer none): Always returns @iter
  */
 DeeModelIter*
 dee_filter_model_append_iter (DeeFilterModel *self,
@@ -437,6 +433,8 @@ dee_filter_model_append_iter (DeeFilterModel *self,
  *
  * This method is usually called when implementing #DeeModelMapFunc or
  * #DeeModelMapNotify methods.
+ *
+ * Return value: (transfer none): Always returns @iter
  */
 DeeModelIter*
 dee_filter_model_prepend_iter (DeeFilterModel *self,
@@ -475,6 +473,8 @@ dee_filter_model_prepend_iter (DeeFilterModel *self,
  *
  * This method is usually called when implementing #DeeModelMapFunc or
  * #DeeModelMapNotify methods.
+ *
+ * Return value: (transfer none): Always returns @iter
  */
 DeeModelIter*
 dee_filter_model_insert_iter (DeeFilterModel *self,                                                        
@@ -501,6 +501,8 @@ dee_filter_model_insert_iter (DeeFilterModel *self,
  *
  * This method is usually called when implementing #DeeModelMapFunc or
  * #DeeModelMapNotify methods.
+ *
+ * Return value: (transfer none): Always returns @iter
  */
 DeeModelIter*
 dee_filter_model_insert_iter_before (DeeFilterModel *self,
@@ -537,6 +539,57 @@ dee_filter_model_insert_iter_before (DeeFilterModel *self,
   return iter;
 }
 
+/**
+ * dee_filter_model_insert_iter_with_original_order:
+ * @self:
+ * @iter:
+ *
+ * Inserts @iter in @self in a way that is consistent with the ordering of the
+ * rows in the original #DeeModel behind @self. THis method assumes that @self
+ * is already ordered this way. If that's not the case then this method has
+ * undefined behaviour.
+ *
+ * This method is mainly intended as a helper for #DeeModelMapNotify functions
+ * of #DeeFilter implementations that creates filter models sorted in
+ * accordance with the original models.
+ *
+ * Return value:(transfer none): Always returns @iter
+ */
+DeeModelIter*
+dee_filter_model_insert_iter_with_original_order (DeeFilterModel *self,
+                                                  DeeModelIter   *iter)
+{
+  DeeFilterModelPrivate *priv;
+  DeeModel              *orig_model;
+  DeeModelIter          *probe, *end;
+
+  g_return_val_if_fail (DEE_IS_FILTER_MODEL (self), NULL);
+  g_return_val_if_fail (iter != NULL, NULL);
+
+  priv = self->priv;
+  orig_model = priv->orig_model;
+
+  /* We find the first row *after* iter which is both in orig_model and the
+   * filter model and insert iter *before* that row  in the filter model.
+   * This should assure the sorting is consistent between the models
+   * (assuming it already was consistent before we started) */
+  probe = dee_model_next (orig_model, iter);
+  end = dee_model_get_last_iter (orig_model);
+  while (probe != end)
+    {
+      if (dee_filter_model_contains (self, probe))
+        {
+          dee_filter_model_insert_iter_before (self, iter, probe);
+          return iter;
+        }
+      probe = dee_model_next (orig_model, probe);
+    }
+
+  /* We made it to the end without finding a suitable place,
+   * so just append it */
+  return dee_filter_model_append_iter (self, iter);
+}
+
 /*
  * Private impl
  */
@@ -611,28 +664,22 @@ on_orig_model_row_changed (DeeFilterModel *self,
 /*
  * DeeModel Interface Implementation
  */
+
 static void
-dee_filter_model_set_column_type (DeeModel *self,
-                                  guint     column,
-                                  GType     type)
+dee_filter_model_set_schema_full (DeeModel *self,
+                                  const gchar* const *schema,
+                                  guint     n_columns)
 {
   g_return_if_fail (DEE_IS_FILTER_MODEL (self));
 
-  g_critical ("You can not set the column type of a DeeFilterModel. "
+  g_critical ("You can not set the schema on a DeeFilterModel. "
               "It will always inherit the ones on the original model");
   return;
 }
 
-static void
-dee_filter_model_set_n_columns (DeeModel *self,
-                                guint     n_columns)
-{
-  g_return_if_fail (DEE_IS_FILTER_MODEL (self));
+/* FALL THROUGH: dee_filter_model_get_schema() */
 
-  g_critical ("You can not set the number of columns on a DeeFilterModel. "
-              "It will always inherit the ones on the original model");
-  return;
-}
+/* FALL THROUGH: dee_filter_model_get_column_schema() */
 
 /* FALL THROUGH: dee_filter_model_get_n_columns() */
 
@@ -647,13 +694,13 @@ dee_filter_model_get_n_rows (DeeModel *self)
   return g_hash_table_size (priv->iter_map) - 1;
 }
 
-/* FALL THROUGH: dee_filter_model_get_column_type() */
+/* FALL THROUGH: dee_filter_model_get_column_schema() */
 
 /* FALL THROUGH: dee_filter_model_clear() */
 
 static DeeModelIter*
-dee_filter_model_prepend_valist (DeeModel *self,
-                                 va_list   args)
+dee_filter_model_prepend_row (DeeModel *self,
+                              GVariant **row_members)
 {
   DeeFilterModelPrivate *priv;
   DeeModelIter          *iter;
@@ -669,14 +716,14 @@ dee_filter_model_prepend_valist (DeeModel *self,
   if (dee_filter_model_is_empty (self))
     {
       priv->ignore_orig_signals = TRUE;
-      iter = dee_model_prepend_valist (priv->orig_model, args);
+      iter = dee_model_prepend_row (priv->orig_model, row_members);
       priv->ignore_orig_signals = FALSE;
     }
   else
     {
       iter = dee_model_get_first_iter (self);
       priv->ignore_orig_signals = TRUE;
-      iter = dee_model_insert_before_valist (priv->orig_model, iter, args);
+      iter = dee_model_insert_row_before (priv->orig_model, iter, row_members);
       priv->ignore_orig_signals = FALSE;
     }
   
@@ -689,8 +736,8 @@ dee_filter_model_prepend_valist (DeeModel *self,
 }
 
 static DeeModelIter*
-dee_filter_model_append_valist (DeeModel *self,
-                               va_list    args)
+dee_filter_model_append_row (DeeModel  *self,
+                             GVariant **row_members)
 {
   DeeFilterModelPrivate *priv;
   DeeModelIter          *iter;
@@ -706,12 +753,13 @@ dee_filter_model_append_valist (DeeModel *self,
   priv->ignore_orig_signals = TRUE;
   if (dee_filter_model_is_empty (self))
     {
-      iter = dee_model_append_valist (priv->orig_model, args);      
+      iter = dee_model_append_row (priv->orig_model, row_members);
     }
   else
     {
       iter = dee_model_get_last_iter (self);
-      iter = dee_model_insert_before_valist (priv->orig_model, iter, args);
+      iter = dee_model_insert_row_before (priv->orig_model,
+                                          iter, row_members);
     }
   priv->ignore_orig_signals = FALSE;
   
@@ -726,9 +774,9 @@ dee_filter_model_append_valist (DeeModel *self,
 /* FALL THROUGH: dee_filter_model_insert_valist() */
 
 static DeeModelIter*
-dee_filter_model_insert_before_valist (DeeModel *self,
-                                       DeeModelIter *iter,
-                                       va_list args)
+dee_filter_model_insert_row_before (DeeModel      *self,
+                                    DeeModelIter  *iter,
+                                    GVariant     **row_members)
 {
   DeeFilterModelPrivate *priv;
   DeeModelIter          *new_iter;
@@ -746,7 +794,7 @@ dee_filter_model_insert_before_valist (DeeModel *self,
     }
   
   priv->ignore_orig_signals = TRUE;
-  new_iter = dee_model_insert_before_valist (priv->orig_model, iter, args);
+  new_iter = dee_model_insert_row_before (priv->orig_model, iter, row_members);
   priv->ignore_orig_signals = FALSE;
   
   seq_iter = g_sequence_insert_before (seq_iter, new_iter);
@@ -916,7 +964,7 @@ dee_filter_model_is_first (DeeModel *self, DeeModelIter *iter)
 
 /* FALL THROUGH: dee_filter_model_is_last()*/
 
-static gint
+static guint
 dee_filter_model_get_position (DeeModel     *self,
                                DeeModelIter *iter)
 {
@@ -934,5 +982,5 @@ dee_filter_model_get_position (DeeModel     *self,
       return -1;
     }
   
-  return g_sequence_iter_get_position (seq_iter);
+  return (guint) ABS(g_sequence_iter_get_position (seq_iter));
 }
