@@ -37,7 +37,7 @@
 
 #include "dee-model.h"
 #include "dee-proxy-model.h"
-#include "dee-versioned-model.h"
+#include "dee-serializable-model.h"
 #include "dee-marshal.h"
 #include "trace-log.h"
 
@@ -45,7 +45,7 @@ static void dee_proxy_model_model_iface_init (DeeModelIface *iface);
 
 G_DEFINE_TYPE_WITH_CODE (DeeProxyModel,
                          dee_proxy_model,
-                         DEE_TYPE_VERSIONED_MODEL,
+                         DEE_TYPE_SERIALIZABLE_MODEL,
                          G_IMPLEMENT_INTERFACE (DEE_TYPE_MODEL,
                                                 dee_proxy_model_model_iface_init));
 
@@ -83,66 +83,56 @@ struct _DeeProxyModelPrivate
 };
 
 #define DEE_PROXY_MODEL_BACK_END(model) (DEE_PROXY_MODEL(model)->priv->back_end)
-#define SUPER_CLASS DEE_VERSIONED_MODEL_CLASS (dee_proxy_model_parent_class)
+#define SUPER_CLASS DEE_SERIALIZABLE_MODEL_CLASS (dee_proxy_model_parent_class)
 
 /*
  * DeeModel forward declarations
  */
-static void           dee_proxy_model_set_column_type (DeeModel *self,
-                                                       guint     column,
-                                                       GType     type);
+static void           dee_proxy_model_set_schema_full (DeeModel           *self,
+                                                       const gchar* const *schema,
+                                                       guint               num_columns);
 
-static void           dee_proxy_model_set_n_columns  (DeeModel *self,
-                                                      guint     n_columns);
+static const gchar* const* dee_proxy_model_get_schema (DeeModel *self,
+                                                       guint    *num_columns);
+
+static const gchar*   dee_proxy_model_get_column_schema (DeeModel *self,
+                                                         guint      column);
                                
 static guint          dee_proxy_model_get_n_columns  (DeeModel *self);
 
 static guint          dee_proxy_model_get_n_rows     (DeeModel *self);
 
-static GType          dee_proxy_model_get_column_type (DeeModel *self,
-                                                       guint      column);
-
 static void           dee_proxy_model_clear          (DeeModel *self);
 
-static DeeModelIter*  dee_proxy_model_append_valist  (DeeModel *self,
-                                                      va_list    args);
+static DeeModelIter*  dee_proxy_model_append_row  (DeeModel  *self,
+                                                   GVariant **row_members);
 
-static DeeModelIter*  dee_proxy_model_prepend_valist  (DeeModel *self,
-                                                       va_list    args);
+static DeeModelIter*  dee_proxy_model_prepend_row  (DeeModel  *self,
+                                                    GVariant **row_members);
 
-static DeeModelIter*  dee_proxy_model_insert_valist  (DeeModel *self,
-                                                      gint       pos,
-                                                      va_list    args);
+static DeeModelIter*  dee_proxy_model_insert_row  (DeeModel  *self,
+                                                   guint      pos,
+                                                   GVariant **row_members);
 
-static DeeModelIter*  dee_proxy_model_insert_before_valist (DeeModel     *self,
-                                                            DeeModelIter *iter,
-                                                            va_list    args);
+static DeeModelIter*  dee_proxy_model_insert_row_before (DeeModel      *self,
+                                                         DeeModelIter  *iter,
+                                                         GVariant     **row_members);
 
 static void           dee_proxy_model_remove         (DeeModel     *self,
                                                       DeeModelIter *iter);
 
-static void           dee_proxy_model_set_valist     (DeeModel       *self,
-                                                      DeeModelIter   *iter,
-                                                      va_list          args);
-
 static void           dee_proxy_model_set_value      (DeeModel       *self,
                                                       DeeModelIter   *iter,
-                                                      guint            column,
-                                                      const GValue    *value);
+                                                      guint           column,
+                                                      GVariant       *value);
 
-static void           dee_proxy_model_set_value_silently (DeeModel       *self,
-                                                          DeeModelIter   *iter,
-                                                          guint            column,
-                                                          const GValue    *value);
+static void           dee_proxy_model_set_row     (DeeModel       *self,
+                                                   DeeModelIter   *iter,
+                                                   GVariant      **row_members);
 
-static void           dee_proxy_model_get_valist     (DeeModel     *self,
+static GVariant*      dee_proxy_model_get_value      (DeeModel     *self,
                                                       DeeModelIter *iter,
-                                                      va_list        args);
-
-static void           dee_proxy_model_get_value      (DeeModel     *self,
-                                                      DeeModelIter *iter,
-                                                      guint          column,
-                                                      GValue        *value);
+                                                      guint          column);
 
 static DeeModelIter* dee_proxy_model_get_first_iter  (DeeModel     *self);
 
@@ -159,13 +149,13 @@ static guchar         dee_proxy_model_get_uchar      (DeeModel     *self,
                                                       DeeModelIter *iter,
                                                       guint          column);
 
-static gint           dee_proxy_model_get_int        (DeeModel     *self,
-                                                      DeeModelIter *iter,
-                                                      guint          column);
+static gint32         dee_proxy_model_get_int32        (DeeModel     *self,
+                                                        DeeModelIter *iter,
+                                                        guint         column);
 
-static guint          dee_proxy_model_get_uint       (DeeModel     *self,
+static guint32        dee_proxy_model_get_uint32     (DeeModel     *self,
                                                       DeeModelIter *iter,
-                                                      guint          column);
+                                                      guint         column);
 
 static gint64         dee_proxy_model_get_int64      (DeeModel     *self,
                                                       DeeModelIter *iter,
@@ -181,7 +171,7 @@ static gdouble        dee_proxy_model_get_double     (DeeModel     *self,
 
 static const gchar*   dee_proxy_model_get_string     (DeeModel     *self,
                                                       DeeModelIter *iter,
-                                                      guint          column);
+                                                      guint         column);
 
 static DeeModelIter* dee_proxy_model_next            (DeeModel     *self,
                                                       DeeModelIter *iter);
@@ -195,12 +185,20 @@ static gboolean       dee_proxy_model_is_first       (DeeModel     *self,
 static gboolean       dee_proxy_model_is_last        (DeeModel     *self,
                                                       DeeModelIter *iter);
 
-static gint           dee_proxy_model_get_position   (DeeModel     *self,
+static guint          dee_proxy_model_get_position   (DeeModel     *self,
                                                       DeeModelIter *iter);
 
-static void           dee_proxy_model_freeze_signals (DeeModel     *self);
+static DeeModelTag*   dee_proxy_model_register_tag   (DeeModel       *self,
+                                                      GDestroyNotify  tag_destroy);
 
-static void           dee_proxy_model_thaw_signals (DeeModel     *self);
+static gpointer       dee_proxy_model_get_tag        (DeeModel       *self,
+                                                      DeeModelIter   *iter,
+                                                      DeeModelTag    *tag);
+
+static void           dee_proxy_model_set_tag        (DeeModel       *self,
+                                                      DeeModelIter   *iter,
+                                                      DeeModelTag    *tag,
+                                                      gpointer        value);
 
 /*
  * Callbacks for relaying signals from the back end model
@@ -215,40 +213,15 @@ static void           on_back_end_row_changed        (DeeProxyModel *self,
                                                       DeeModelIter  *iter);
 
 /*
- * Overrides for DeeVersionedModel
+ * Overrides for DeeSerializableModel
  */
-static DeeSeqnumState
-                 dee_proxy_model_check_seqnums    (DeeModel *self,
-                                                   GArray *seqnums);
 
-static DeeSeqnumState
-                 dee_proxy_model_check_seqnum     (DeeModel     *self,
-                                                   guint64       seqnum);
+static guint64   dee_proxy_model_get_seqnum  (DeeModel     *self);
 
-static guint64   dee_proxy_model_inc_seqnum       (DeeModel     *self,
-                                                   guint         pos);
+static void      dee_proxy_model_set_seqnum  (DeeModel     *self,
+                                              guint64       seqnum);
 
-static void      dee_proxy_model_set_seqnum       (DeeModel     *self,
-                                                   guint         pos,
-                                                   guint64       seqnum);
-
-static guint64   dee_proxy_model_append_next_seqnum  (DeeModel     *self);
-
-static guint64   dee_proxy_model_prepend_next_seqnum (DeeModel     *self);
-
-static guint64   dee_proxy_model_insert_next_seqnum  (DeeModel     *self,
-                                                      guint         pos);
-
-static guint64   dee_proxy_model_get_seqnum       (DeeModel     *self,
-                                                   guint         pos);
-
-static guint64   dee_proxy_model_get_last_seqnum  (DeeModel     *self);
-
-static void      dee_proxy_model_set_last_seqnum  (DeeModel     *self,
-                                                   guint64       seqnum);
-
-static void      dee_proxy_model_remove_seqnum    (DeeModel     *self,
-                                                   guint         pos);
+static guint64   dee_proxy_model_inc_seqnum  (DeeModel     *self);
 
 /* GObject Init */
 static void
@@ -283,7 +256,7 @@ dee_proxy_model_constructed (GObject *object)
                 "the DeeProxyModel upon creation.");
     return;
   }
-  
+
   /* Connect to signals on the back-end model so we can relay them */
   if (priv->proxy_signals)
     {
@@ -351,7 +324,7 @@ static void
 dee_proxy_model_class_init (DeeProxyModelClass *klass)
 {
   GObjectClass           *obj_class = G_OBJECT_CLASS (klass);
-  DeeVersionedModelClass *dvm_class = DEE_VERSIONED_MODEL_CLASS (klass);
+  DeeSerializableModelClass *dvm_class = DEE_SERIALIZABLE_MODEL_CLASS (klass);
   GParamSpec             *pspec;
 
   obj_class->finalize     = dee_proxy_model_finalize;
@@ -359,17 +332,9 @@ dee_proxy_model_class_init (DeeProxyModelClass *klass)
   obj_class->set_property = dee_proxy_model_set_property;
   obj_class->get_property = dee_proxy_model_get_property;
 
-  dvm_class->remove_seqnum       = dee_proxy_model_remove_seqnum;
-  dvm_class->get_last_seqnum     = dee_proxy_model_get_last_seqnum;
-  dvm_class->set_last_seqnum     = dee_proxy_model_set_last_seqnum;
   dvm_class->get_seqnum          = dee_proxy_model_get_seqnum;
-  dvm_class->insert_next_seqnum  = dee_proxy_model_insert_next_seqnum;
-  dvm_class->prepend_next_seqnum = dee_proxy_model_prepend_next_seqnum;
-  dvm_class->append_next_seqnum  = dee_proxy_model_append_next_seqnum;
   dvm_class->set_seqnum          = dee_proxy_model_set_seqnum;
   dvm_class->inc_seqnum          = dee_proxy_model_inc_seqnum;
-  dvm_class->check_seqnum        = dee_proxy_model_check_seqnum;
-  dvm_class->check_seqnums       = dee_proxy_model_check_seqnums;  
 
   /**
    * DeeProxyModel:back-end:
@@ -405,29 +370,27 @@ dee_proxy_model_class_init (DeeProxyModelClass *klass)
 static void
 dee_proxy_model_model_iface_init (DeeModelIface *iface)
 {
-  iface->set_column_type      = dee_proxy_model_set_column_type;
-  iface->set_n_columns        = dee_proxy_model_set_n_columns;
+  iface->set_schema_full      = dee_proxy_model_set_schema_full;
+  iface->get_schema           = dee_proxy_model_get_schema;
+  iface->get_column_schema    = dee_proxy_model_get_column_schema;
   iface->get_n_columns        = dee_proxy_model_get_n_columns;
   iface->get_n_rows           = dee_proxy_model_get_n_rows;
-  iface->get_column_type      = dee_proxy_model_get_column_type;
   iface->clear                = dee_proxy_model_clear;
-  iface->prepend_valist       = dee_proxy_model_prepend_valist;
-  iface->append_valist        = dee_proxy_model_append_valist;
-  iface->insert_valist        = dee_proxy_model_insert_valist;
-  iface->insert_before_valist = dee_proxy_model_insert_before_valist;
+  iface->prepend_row          = dee_proxy_model_prepend_row;
+  iface->append_row           = dee_proxy_model_append_row;
+  iface->insert_row           = dee_proxy_model_insert_row;
+  iface->insert_row_before    = dee_proxy_model_insert_row_before;
   iface->remove               = dee_proxy_model_remove;
-  iface->set_valist           = dee_proxy_model_set_valist;
   iface->set_value            = dee_proxy_model_set_value;
-  iface->set_value_silently   = dee_proxy_model_set_value_silently;
-  iface->get_valist           = dee_proxy_model_get_valist;
+  iface->set_row              = dee_proxy_model_set_row;
   iface->get_value            = dee_proxy_model_get_value;
   iface->get_first_iter       = dee_proxy_model_get_first_iter;
   iface->get_last_iter        = dee_proxy_model_get_last_iter;
   iface->get_iter_at_row      = dee_proxy_model_get_iter_at_row;
   iface->get_bool             = dee_proxy_model_get_bool;
   iface->get_uchar            = dee_proxy_model_get_uchar;
-  iface->get_int              = dee_proxy_model_get_int;
-  iface->get_uint             = dee_proxy_model_get_uint;
+  iface->get_int32            = dee_proxy_model_get_int32;
+  iface->get_uint32           = dee_proxy_model_get_uint32;
   iface->get_int64            = dee_proxy_model_get_int64;
   iface->get_uint64           = dee_proxy_model_get_uint64;
   iface->get_double           = dee_proxy_model_get_double;
@@ -437,8 +400,9 @@ dee_proxy_model_model_iface_init (DeeModelIface *iface)
   iface->is_first             = dee_proxy_model_is_first;
   iface->is_last              = dee_proxy_model_is_last;
   iface->get_position         = dee_proxy_model_get_position;
-  iface->freeze_signals       = dee_proxy_model_freeze_signals;
-  iface->thaw_signals         = dee_proxy_model_thaw_signals;
+  iface->register_tag         = dee_proxy_model_register_tag;
+  iface->get_tag              = dee_proxy_model_get_tag;
+  iface->set_tag              = dee_proxy_model_set_tag;
 }
 
 static void
@@ -459,22 +423,32 @@ dee_proxy_model_init (DeeProxyModel *model)
  * DeeModel Interface Implementation
  */
 static void
-dee_proxy_model_set_column_type (DeeModel *self,
-                                 guint     column,
-                                 GType     type)
+dee_proxy_model_set_schema_full (DeeModel           *self,
+                                 const gchar* const *schema,
+                                 guint               num_columns)
 {
   g_return_if_fail (DEE_IS_PROXY_MODEL (self));
 
-  dee_model_set_column_type (DEE_PROXY_MODEL_BACK_END (self), column, type);
+  dee_model_set_schema_full (DEE_PROXY_MODEL_BACK_END (self),
+                             schema, num_columns);
 }
 
-static void
-dee_proxy_model_set_n_columns (DeeModel *self,
-                               guint     n_columns)
+static const gchar* const*
+dee_proxy_model_get_schema (DeeModel *self,
+                            guint    *num_columns)
 {
-  g_return_if_fail (DEE_IS_PROXY_MODEL (self));
+  g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), 0);
 
-  dee_model_set_n_columns (DEE_PROXY_MODEL_BACK_END (self), n_columns);
+  return dee_model_get_schema (DEE_PROXY_MODEL_BACK_END (self), num_columns);
+}
+
+static const gchar*
+dee_proxy_model_get_column_schema (DeeModel *self,
+                                   guint     column)
+{
+  g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), NULL);
+
+  return dee_model_get_column_schema (DEE_PROXY_MODEL_BACK_END (self), column);
 }
 
 static guint
@@ -493,15 +467,6 @@ dee_proxy_model_get_n_rows (DeeModel *self)
   return dee_model_get_n_rows (DEE_PROXY_MODEL_BACK_END (self));
 }
 
-static GType
-dee_proxy_model_get_column_type (DeeModel *self,
-                                 guint      column)
-{
-  g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), G_TYPE_NONE);
-
-  return dee_model_get_column_type (DEE_PROXY_MODEL_BACK_END (self), column);
-}
-
 static void
 dee_proxy_model_clear (DeeModel *self)
 {
@@ -511,41 +476,43 @@ dee_proxy_model_clear (DeeModel *self)
 }
 
 static DeeModelIter*
-dee_proxy_model_prepend_valist (DeeModel *self,
-                                va_list    args)
+dee_proxy_model_prepend_row (DeeModel  *self,
+                             GVariant **row_members)
 {
   g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), NULL);
 
-  return dee_model_prepend_valist (DEE_PROXY_MODEL_BACK_END (self), args);
+  return dee_model_prepend_row (DEE_PROXY_MODEL_BACK_END (self), row_members);
 }
 
 static DeeModelIter*
-dee_proxy_model_append_valist (DeeModel *self,
-                               va_list    args)
+dee_proxy_model_append_row (DeeModel  *self,
+                            GVariant **row_members)
 {
   g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), NULL);
 
-  return dee_model_append_valist (DEE_PROXY_MODEL_BACK_END (self), args);
+  return dee_model_append_row (DEE_PROXY_MODEL_BACK_END (self), row_members);
 }
 
 static DeeModelIter*
-dee_proxy_model_insert_valist (DeeModel *self,
-                               gint       pos,
-                               va_list    args)
+dee_proxy_model_insert_row (DeeModel  *self,
+                            guint      pos,
+                            GVariant **row_members)
 {
   g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), NULL);
 
-  return dee_model_insert_valist (DEE_PROXY_MODEL_BACK_END (self), pos, args);
+  return dee_model_insert_row (DEE_PROXY_MODEL_BACK_END (self),
+                               pos, row_members);
 }
 
 static DeeModelIter*
-dee_proxy_model_insert_before_valist (DeeModel *self,
-                                      DeeModelIter *iter,
-                                      va_list args)
+dee_proxy_model_insert_row_before (DeeModel      *self,
+                                   DeeModelIter  *iter,
+                                   GVariant     **row_members)
 {
   g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), NULL);
 
-  return dee_model_insert_before_valist (DEE_PROXY_MODEL_BACK_END (self), iter, args);
+  return dee_model_insert_row_before (DEE_PROXY_MODEL_BACK_END (self),
+                                      iter, row_members);
 }
 
 static void
@@ -558,57 +525,34 @@ dee_proxy_model_remove (DeeModel     *self,
 }
 
 static void
-dee_proxy_model_set_valist (DeeModel       *self,
-                            DeeModelIter   *iter,
-                            va_list         args)
+dee_proxy_model_set_row (DeeModel       *self,
+                         DeeModelIter   *iter,
+                         GVariant      **row_members)
 {
   g_return_if_fail (DEE_IS_PROXY_MODEL (self));
 
-  dee_model_set_valist (DEE_PROXY_MODEL_BACK_END (self), iter, args);
+  dee_model_set_row (DEE_PROXY_MODEL_BACK_END (self), iter, row_members);
 }
 
 static void
 dee_proxy_model_set_value (DeeModel      *self,
                            DeeModelIter  *iter,
                            guint          column,
-                           const GValue  *value)
+                           GVariant      *value)
 {
   g_return_if_fail (DEE_IS_PROXY_MODEL (self));
 
   dee_model_set_value (DEE_PROXY_MODEL_BACK_END (self), iter, column, value);
 }
 
-static void
-dee_proxy_model_set_value_silently (DeeModel      *self,
-                                    DeeModelIter  *iter,
-                                    guint          column,
-                                    const GValue  *value)
-{
-  g_return_if_fail (DEE_IS_PROXY_MODEL (self));
-
-  dee_model_set_value_silently (DEE_PROXY_MODEL_BACK_END (self),
-                                iter, column, value);
-}
-
-static void
-dee_proxy_model_get_valist (DeeModel       *self,
-                            DeeModelIter   *iter,
-                            va_list         args)
-{
-  g_return_if_fail (DEE_IS_PROXY_MODEL (self));
-
-  dee_model_get_valist (DEE_PROXY_MODEL_BACK_END (self), iter, args);
-}
-
-static void
+static GVariant*
 dee_proxy_model_get_value (DeeModel     *self,
                            DeeModelIter *iter,
-                           guint         column,
-                           GValue        *value)
+                           guint         column)
 {
-  g_return_if_fail (DEE_IS_PROXY_MODEL (self));
+  g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), NULL);
 
-  dee_model_get_value (DEE_PROXY_MODEL_BACK_END (self), iter, column, value);
+  return dee_model_get_value (DEE_PROXY_MODEL_BACK_END (self), iter, column);
 }
 
 static gboolean
@@ -631,24 +575,24 @@ dee_proxy_model_get_uchar (DeeModel      *self,
   return dee_model_get_uchar (DEE_PROXY_MODEL_BACK_END (self), iter, column);
 }
 
-static gint
-dee_proxy_model_get_int (DeeModel        *self,
-                         DeeModelIter    *iter,
-                         guint            column)
+static gint32
+dee_proxy_model_get_int32 (DeeModel        *self,
+                           DeeModelIter    *iter,
+                           guint            column)
 {
   g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), 0);
 
-  return dee_model_get_int (DEE_PROXY_MODEL_BACK_END (self), iter, column);
+  return dee_model_get_int32 (DEE_PROXY_MODEL_BACK_END (self), iter, column);
 }
 
-static guint
-dee_proxy_model_get_uint (DeeModel      *self,
-                          DeeModelIter  *iter,
-                          guint          column)
+static guint32
+dee_proxy_model_get_uint32 (DeeModel      *self,
+                            DeeModelIter  *iter,
+                            guint          column)
 {
   g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), 0);
 
-  return dee_model_get_uint (DEE_PROXY_MODEL_BACK_END (self), iter, column);
+  return dee_model_get_uint32 (DEE_PROXY_MODEL_BACK_END (self), iter, column);
 }
 
 
@@ -753,7 +697,7 @@ dee_proxy_model_is_last (DeeModel     *self,
   return dee_model_is_last (DEE_PROXY_MODEL_BACK_END (self), iter);
 }
 
-static gint
+static guint
 dee_proxy_model_get_position (DeeModel     *self,
                               DeeModelIter *iter)
 {
@@ -762,20 +706,34 @@ dee_proxy_model_get_position (DeeModel     *self,
   return dee_model_get_position (DEE_PROXY_MODEL_BACK_END (self), iter);
 }
 
-static void
-dee_proxy_model_freeze_signals (DeeModel     *self)
+static DeeModelTag*
+dee_proxy_model_register_tag    (DeeModel       *self,
+                                 GDestroyNotify  tag_destroy)
 {
-  g_return_if_fail (DEE_IS_PROXY_MODEL (self));
+  g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), NULL);
 
-  dee_model_freeze_signals (DEE_PROXY_MODEL_BACK_END (self));
+  return dee_model_register_tag (DEE_PROXY_MODEL_BACK_END (self), tag_destroy);
+}
+
+static gpointer
+dee_proxy_model_get_tag (DeeModel       *self,
+                         DeeModelIter   *iter,
+                         DeeModelTag    *tag)
+{
+  g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), NULL);
+
+  return dee_model_get_tag (DEE_PROXY_MODEL_BACK_END (self), iter, tag);
 }
 
 static void
-dee_proxy_model_thaw_signals (DeeModel     *self)
+dee_proxy_model_set_tag (DeeModel       *self,
+                         DeeModelIter   *iter,
+                         DeeModelTag    *tag,
+                         gpointer        value)
 {
   g_return_if_fail (DEE_IS_PROXY_MODEL (self));
 
-  dee_model_thaw_signals (DEE_PROXY_MODEL_BACK_END (self));
+  return dee_model_set_tag (DEE_PROXY_MODEL_BACK_END (self), iter, tag, value);
 }
 
 /*
@@ -804,43 +762,10 @@ on_back_end_row_changed (DeeProxyModel *self,
 }
 
 /*
- * Overrides for DeeVersionedModel
+ * Overrides for DeeSerializableModel
  */
-static DeeSeqnumState
-dee_proxy_model_check_seqnums (DeeModel *self,
-                               GArray *seqnums)
-{
-  DeeProxyModelPrivate *priv;
-  
-  g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), 0);
-  
-  priv = DEE_PROXY_MODEL (self)->priv;
-
-  if (priv->inherit_seqnums)
-    return dee_versioned_model_check_seqnums (priv->back_end, seqnums);
-  else
-    return SUPER_CLASS->check_seqnums (priv->back_end, seqnums);
-}
-
-static DeeSeqnumState
-dee_proxy_model_check_seqnum (DeeModel     *self,
-                              guint64       seqnum)
-{
-  DeeProxyModelPrivate *priv;
-  
-  g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), 0);
-  
-  priv = DEE_PROXY_MODEL (self)->priv;
-
-  if (priv->inherit_seqnums)
-    return dee_versioned_model_check_seqnum (priv->back_end, seqnum);
-  else
-    return SUPER_CLASS->check_seqnum (priv->back_end, seqnum);
-}
-
 static guint64
-dee_proxy_model_inc_seqnum (DeeModel     *self,
-                            guint         pos)
+dee_proxy_model_get_seqnum (DeeModel     *self)
 {
   DeeProxyModelPrivate *priv;
   
@@ -849,15 +774,14 @@ dee_proxy_model_inc_seqnum (DeeModel     *self,
   priv = DEE_PROXY_MODEL (self)->priv;
 
   if (priv->inherit_seqnums)
-    return dee_versioned_model_inc_seqnum (priv->back_end, pos);
+    return dee_serializable_model_get_seqnum (priv->back_end);
   else
-    return SUPER_CLASS->inc_seqnum (priv->back_end, pos);
+    return SUPER_CLASS->get_seqnum (priv->back_end);
 }
 
 static void
-dee_proxy_model_set_seqnum (DeeModel     *self,
-                            guint         pos,
-                            guint64       seqnum)
+dee_proxy_model_set_seqnum  (DeeModel     *self,
+                             guint64       seqnum)
 {
   DeeProxyModelPrivate *priv;
   
@@ -866,13 +790,13 @@ dee_proxy_model_set_seqnum (DeeModel     *self,
   priv = DEE_PROXY_MODEL (self)->priv;
 
   if (priv->inherit_seqnums)
-    dee_versioned_model_set_seqnum (priv->back_end, pos, seqnum);
+    dee_serializable_model_set_seqnum (priv->back_end, seqnum);
   else
-    SUPER_CLASS->set_seqnum (priv->back_end, pos, seqnum);
+    return SUPER_CLASS->set_seqnum (priv->back_end, seqnum);
 }
 
 static guint64
-dee_proxy_model_append_next_seqnum (DeeModel *self)
+dee_proxy_model_inc_seqnum (DeeModel     *self)
 {
   DeeProxyModelPrivate *priv;
   
@@ -881,101 +805,7 @@ dee_proxy_model_append_next_seqnum (DeeModel *self)
   priv = DEE_PROXY_MODEL (self)->priv;
 
   if (priv->inherit_seqnums)
-    return dee_versioned_model_append_next_seqnum (priv->back_end);
+    return dee_serializable_model_inc_seqnum (priv->back_end);
   else
-    return SUPER_CLASS->append_next_seqnum (priv->back_end);
-}
-
-static guint64
-dee_proxy_model_prepend_next_seqnum (DeeModel *self)
-{
-  DeeProxyModelPrivate *priv;
-  
-  g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), 0);
-  
-  priv = DEE_PROXY_MODEL (self)->priv;
-
-  if (priv->inherit_seqnums)
-    return dee_versioned_model_prepend_next_seqnum (priv->back_end);
-  else
-    return SUPER_CLASS->prepend_next_seqnum (priv->back_end);
-}
-
-static guint64
-dee_proxy_model_insert_next_seqnum  (DeeModel     *self,
-                                     guint         pos)
-{
-  DeeProxyModelPrivate *priv;
-  
-  g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), 0);
-  
-  priv = DEE_PROXY_MODEL (self)->priv;
-
-  if (priv->inherit_seqnums)
-    return dee_versioned_model_insert_next_seqnum (priv->back_end, pos);
-  else
-    return SUPER_CLASS->insert_next_seqnum (priv->back_end, pos);
-}
-
-static guint64
-dee_proxy_model_get_seqnum (DeeModel     *self,
-                            guint         pos)
-{
-  DeeProxyModelPrivate *priv;
-  
-  g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), 0);
-  
-  priv = DEE_PROXY_MODEL (self)->priv;
-
-  if (priv->inherit_seqnums)
-    return dee_versioned_model_get_seqnum (priv->back_end, pos);
-  else
-    return SUPER_CLASS->get_seqnum (priv->back_end, pos);
-}
-
-static guint64
-dee_proxy_model_get_last_seqnum (DeeModel     *self)
-{
-  DeeProxyModelPrivate *priv;
-  
-  g_return_val_if_fail (DEE_IS_PROXY_MODEL (self), 0);
-  
-  priv = DEE_PROXY_MODEL (self)->priv;
-
-  if (priv->inherit_seqnums)
-    return dee_versioned_model_get_last_seqnum (priv->back_end);
-  else
-    return SUPER_CLASS->get_last_seqnum (priv->back_end);
-}
-
-static void
-dee_proxy_model_set_last_seqnum  (DeeModel     *self,
-                                  guint64       seqnum)
-{
-  DeeProxyModelPrivate *priv;
-  
-  g_return_if_fail (DEE_IS_PROXY_MODEL (self));
-  
-  priv = DEE_PROXY_MODEL (self)->priv;
-
-  if (priv->inherit_seqnums)
-    dee_versioned_model_set_last_seqnum (priv->back_end, seqnum);
-  else
-    return SUPER_CLASS->set_last_seqnum (priv->back_end, seqnum);
-}
-
-static void
-dee_proxy_model_remove_seqnum (DeeModel     *self,
-                               guint         pos)
-{
-  DeeProxyModelPrivate *priv;
-  
-  g_return_if_fail (DEE_IS_PROXY_MODEL (self));
-  
-  priv = DEE_PROXY_MODEL (self)->priv;
-
-  if (priv->inherit_seqnums)
-    return dee_versioned_model_remove_seqnum (priv->back_end, pos);
-  else
-    return SUPER_CLASS->remove_seqnum (priv->back_end, pos);
+    return SUPER_CLASS->inc_seqnum (priv->back_end);
 }
